<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:java="http://xml.apache.org/xslt/java"
	xmlns:util="xalan://org.greenstone.gsdl3.util.XSLTUtil"
	xmlns:gslib="http://www.greenstone.org/skinning"
	xmlns:gsf="http://www.greenstone.org/greenstone3/schema/ConfigFormat"	
	extension-element-prefixes="java util"
	exclude-result-prefixes="java util">


	<!-- use the 'main' layout -->
	<xsl:import href="layouts/main.xsl"/>

	<xsl:variable name="groupPath"><xsl:value-of select="/page/pageRequest/paramList/param[@name='group']/@value"/></xsl:variable>
	<!-- set page title -->
	<xsl:template name="pageTitle">SPARQL Query</xsl:template>

	<!-- set page breadcrumbs -->
	<xsl:template name="breadcrumbs"><gslib:siteLink/><gslib:rightArrow/>
	<xsl:if test="$groupPath != ''">
	  <xsl:for-each select="/page/pageResponse/pathList/group">
	    <xsl:sort data-type="number" select="@position"/>
	      <a>
		<xsl:attribute name="href"><gslib:groupHref path="{@path}"/></xsl:attribute>				
		<xsl:attribute name="title"><gslib:groupName path="{@path}"/></xsl:attribute>
		<gslib:groupName path="{@path}"/>
	      </a>
	      <gslib:rightArrow/>
	  </xsl:for-each>
	</xsl:if>
	<a href="{$library_name}/collection/{$collName}/page/about"><gslib:collectionName/></a>
	</xsl:template>

	<!-- the page content -->
	<xsl:template match="/page">
	  <!--
	  <link rel="stylesheet" href="sites/{$site_name}/collect/{$collName}/style/fuseki.css" type="text/css" />
	  -->
	  <gsf:script src="sites/{$site_name}/collect/{$collName}/js/jquery.show-more.js"/>
	  <gsf:script src="sites/{$site_name}/collect/{$collName}/js/eurovision.js"/>
	  <gsf:script src="sites/{$site_name}/collect/{$collName}/js/dataviz.js"/>

	  <!--
	  <link rel="stylesheet" href="sites/{$site_name}/collect/{$collName}/css/eurovision.css" type="text/css" />
	  -->
	  <gsf:style src="sites/{$site_name}/collect/{$collName}/css/eurovision.css" />
	  <gsf:style src="sites/{$site_name}/collect/{$collName}/css/dataviz.css" />
	  <style>
	    div          { padding-left: 12px; padding-right: 12px; padding-top: 6px; padding-bottom: 6px;}
	    div.showmore { padding-left:  0px; padding-right:  0px; padding-top: 0px; padding-bottom: 0px;}

	    p  { padding-top: 6px; padding-bottom: 6px;}
	    a  { text-decoration: underline; }
	    li { padding-bottom: 6px; margin-bottom: 6px; }
	  </style>

	  <div>
	    <h2>Query the Linked-Data and let its values Sparkle (aka SPARQL)</h2>

	    <p>
	      Using this page you can query the linked-data in the Greenstone3 <i>eurovision</i> triplestore
	      graph using <a href="https://en.wikipedia.org/wiki/SPARQL">SPARQL</a> query syntax.
	      The default query provided in the text box below is effectively issuing a query that
	      returns everything stored (a bit like matching with the wildcard '*' that some text-based search
	      systems have), limited to show just the first 500 entries that matched.
	    </p>
	    
	    <p>
	      Use the default query to get a feel for the data that is
	      stored.  Feel free to experiment with the syntax to vary
	      what is returned.  For example, to view other "slices"
	      of the rows of data returned, change the values of
	      OFFSET and LIMIT.  OFFSET controls how many result rows
	      are skipped over before rows are selected.  LIMIT
	      controls how many rows are then taken.
	      
	    </p>

	    <div id="sparql-show-more" class="showmore">
	      <p>
		Alternatively, you might be interested in experimenting
		with forms of access to the triplestore through the more powerful, but complex:
		<ul>
		  <li>
		    <a target="_blank" href="https://so-we-must-think.space/greenstone3-lod3/dataset.html?tab=query&amp;ds=/greenstone">Fuseki interface</a>
		  </li>
		</ul>
	      </p>
	      <p>
		Or else export the entire triplestore to work on it with whichever Linked Data tools are your preference:
		<ul>
		  <li>
		    <a href="/greenstone3-lod3/greenstone/data" download="greenstone3-eurovision-lod.trig">Full Export in TriG Syntax</a>
		  </li>
		  <li>
		    <a href="/greenstone3-lod3/greenstone/data" download="greenstone3-eurovision-lod.ttl">Full Export in Turtle Syntax</a>
		  </li>
		</ul>
	      </p>
	      
	    </div>
	    <gsf:script>
	      $('#sparql-show-more').showMore({
	        minheight: 0,
		buttontxtmore:"show more ...",
		buttontxtless:"... show less"
	      });
	    </gsf:script>	    
	  
	  </div>	  
	  <div>
	    <h2>SPARQL Query</h2>
	    <!--
	    <p>
	      Dataset: /greenstone<br/>
	      Graph: eurovision
	    </p>
	    -->
	    
	  </div>

	  <xsl:variable name="graphURI">https://so-we-must-think.space<xsl:value-of select="$siteURL"/><xsl:value-of select="$library_name"/>/collection/<xsl:value-of select="$collName"/></xsl:variable>
	  
	  <div class="moreindent">
	    <form id="sample-sparql-query-form" action="/greenstone3-lod3/greenstone/query" target="_blank"
		  method="POST" accept-charset="UTF-8">

              <textarea id="sample-sparql-query" name="query" style="width:100%; min-width:100%; max-width: 100%;" rows="14">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;

SELECT ?subject ?predicate ?object WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    ?subject ?predicate ?object.
  }
}
OFFSET 0 LIMIT 500
</xsl:text>
	      </textarea>
	      
	      <div style="display:none;">
		<div>
		  Output:
		  <select name="output">
		    <option value="text">Text</option>
		    <option value="json">JSON</option>
		    <option value="xml" selected="selected">XML</option>
		    <option value="csv">CSV</option>
		    <option value="tsv">TSV</option>
		  </select>
		</div>
		
		<div>
		  If XML output, add XSLT style sheet (blank for none): 
		  <select name="stylesheet">
		    <option value=""></option>
		    <option value="/greenstone3-lod/xml-to-html.xsl">xml-to-html</option>
		    <option value="/greenstone3-lod/xml-to-html-links.xsl" selected="selected">xml-to-html-links</option>
		    <option value="/greenstone3-lod/xml-to-html-plain.xsl">xml-to-html-plain</option>
		  </select>
		</div>
		
		<div>
		  <input type="checkbox" name="force-accept" value="text/plain" />
		  Force the accept header to <tt>text/plain</tt> regardless.
		</div>
	      </div>
	      
	      <div>
		<input type="submit" value="Get Results" />
	      </div>
	      
	    </form>
	  </div>

	  <div>
	    <h2>Sample Queries</h2>

	    <p>
	      The sample queries provided below gradually increase in
	      the query syntax complexity used.  They are intended
	      primarily for illustrative purposes, to introduce a user
	      unfamiliar to various aspects of SPARQL query syntax.
	      Click on one of the <i>Load query above</i> buttons to load
	      the relevant query syntax into the SPARQL Query textbox
	      above, then press <i>Get Results</i> to initiate the
	      query.
	    </p>
	      
	    <div id="ssq-restrict-year" style="display: none;">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;

SELECT ?subject ?predicate ?object WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    ?subject ?predicate ?object.
    ?subject gsdlextracted:Year "2000".    
  }
}
OFFSET 0 LIMIT 200
</xsl:text>
	    </div>

	    <div id="ssq-restrict-year-then-filter" style="display: none;">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;

SELECT ?subject ?predicate ?object WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    ?subject ?predicate ?object.
    ?subject gsdlextracted:Year "2000".    

    # The following removes a triple that stores JSON data that
    # is voluminous but not very interesting to look at
    FILTER (?predicate != gsdlextracted:JSON).
  }
}
OFFSET 0 LIMIT 200
</xsl:text>
	    </div>
	    

	    <div id="ssq-restrict-country-and-year" style="display: none;">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;
PREFIX xsd: &lt;http://www.w3.org/2001/XMLSchema#&gt;
  
SELECT ?subject ?predicate ?object WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    ?subject ?predicate ?object

    # The following removes a triple that stores JSON data that
    # is voluminous but not very interesting to look at
    FILTER (?predicate != gsdlextracted:JSON).

    ?subject gsdlextracted:Year ?year.
    ?subject gsdlextracted:Country ?country.    
    FILTER (?country = "France").
    FILTER (xsd:integer(?year) >= 1996 &amp;&amp; xsd:integer(?year) &lt;= 2000).

  }
}
OFFSET 0 LIMIT 200
</xsl:text>
	    </div>


	    <div id="ssq-esc-entrants" style="display: none;">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;
PREFIX xsd: &lt;http://www.w3.org/2001/XMLSchema#&gt;
PREFIX dc: &lt;http://purl.org/dc/elements/1.1/&gt;

SELECT ?esc_entrant_uri ?country ?year WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    ?esc_entrant_uri dc:Relation.isPartOf &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;.

    ?esc_entrant_uri gsdlextracted:Year ?year.
    ?esc_entrant_uri gsdlextracted:Country ?country.    
  }
}
ORDER BY ASC(?year) ?country
</xsl:text>
	    </div>


	    <div id="ssq-jury-votes-totals-1975" style="display: none;">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;
PREFIX xsd: &lt;http://www.w3.org/2001/XMLSchema#&gt;
PREFIX dc: &lt;http://purl.org/dc/elements/1.1/&gt;

SELECT ?country ?year ?total WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    ?esc_entrant_uri dc:Relation.isPartOf &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;.

    ?esc_entrant_uri gsdlextracted:SingVotesTotal ?total.
    BIND(xsd:integer(?total) AS ?total_int).
    
    ?esc_entrant_uri gsdlextracted:Year ?year
      FILTER(xsd:integer(?year) = 1975).
    ?esc_entrant_uri gsdlextracted:Country ?country.    
  }
}
ORDER BY DESC(?total_int)
</xsl:text>
	    </div>

	    
	    <div id="ssq-jury-votes-1975" style="display: none;">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;
PREFIX xsd: &lt;http://www.w3.org/2001/XMLSchema#&gt;
PREFIX dc: &lt;http://purl.org/dc/elements/1.1/&gt;

SELECT ?country ?year ?jury_country ?jury_vote WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    ?esc_entrant_uri dc:Relation.isPartOf &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;.

    ?esc_entrant_uri ?gsdlextracted_jury_country_uri ?jury_vote.
    BIND(STRENDS(str(?gsdlextracted_jury_country_uri),"-J") AS ?is_jury_vote)
    FILTER (?is_jury_vote).
    BIND(REPLACE(str(?gsdlextracted_jury_country_uri), ".*#(.*?)-J$", "$1") AS ?jury_country).

    ?esc_entrant_uri gsdlextracted:Year ?year
      FILTER(xsd:integer(?year) = 1975).
    ?esc_entrant_uri gsdlextracted:Country ?country.

      
  }
}
ORDER BY ASC(?year) ?country  
</xsl:text>
	    </div>


	    <div id="ssq-winning-totals" style="display: none;">
<!-- -->
<xsl:text>
PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;
PREFIX xsd: &lt;http://www.w3.org/2001/XMLSchema#&gt;
PREFIX dc: &lt;http://purl.org/dc/elements/1.1/&gt;

SELECT ?year ?country ?total_max WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {

    { 
      SELECT ?year (STR(MAX(?total_int_inner)) as ?total_max) WHERE {
      
        ?esc_entrant_uri gsdlextracted:VoteGrandTotal ?total.
        BIND(xsd:integer(?total) AS ?total_int_inner).
    
        ?esc_entrant_uri gsdlextracted:Year ?year.
      }
      GROUP BY ?year
     }

    ?esc_entrant_uri gsdlextracted:VoteGrandTotal ?total_max.    
    ?esc_entrant_uri gsdlextracted:Year ?year.
    ?esc_entrant_uri gsdlextracted:Country ?country.

    BIND(xsd:integer(?total_max) as ?total_int).    
  }
}
ORDER BY DESC(?total_int)
</xsl:text>
	    </div>


	    <div id="ssq-winning-freq-count" style="display: none;">
<!-- -->
<xsl:text>

PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;
PREFIX xsd: &lt;http://www.w3.org/2001/XMLSchema#&gt;
PREFIX dc: &lt;http://purl.org/dc/elements/1.1/&gt;


SELECT ?country (STR(COUNT(?country)) as ?number_of_vote_wins) WHERE {
  GRAPH &lt;</xsl:text><xsl:value-of select="$graphURI"/><xsl:text>&gt;  {
    { 
      SELECT ?year (STR(MAX(?total_int)) as ?total_max) WHERE {      
        ?esc_entrant_uri gsdlextracted:VoteGrandTotal ?total.
        BIND(xsd:integer(?total) AS ?total_int).
    
        ?esc_entrant_uri gsdlextracted:Year ?year.
      }
      GROUP BY ?year
    }

    ?esc_entrant_uri gsdlextracted:VoteGrandTotal ?total_max.    
    ?esc_entrant_uri gsdlextracted:Year ?year.
    ?esc_entrant_uri gsdlextracted:Country ?country.    
  }
}
GROUP BY ?country
ORDER BY DESC(?number_of_vote_wins)
</xsl:text>
	    </div>




	    <div id="ssq-country-info" style="display: none;">
<!-- 
  Adapted from:
     https://stackoverflow.com/questions/40069644/retrieve-information-about-all-european-countries
-->
<xsl:text>

PREFIX gsdlextracted: &lt;http://greenstone.org/gsdlextracted#&gt;

PREFIX  dbo:  &lt;http://dbpedia.org/ontology/&gt;
PREFIX  dbp:  &lt;http://dbpedia.org/property/&gt;
PREFIX  dct:  &lt;http://purl.org/dc/terms/&gt;
#PREFIX  geo:  &lt;http://www.opengis.net/ont/geosparql#&gt;

PREFIX geo: &lt;http://www.w3.org/2003/01/geo/wgs84_pos#&gt;

SELECT DISTINCT ?country_uri ?capital ?area ?populationTotal ?lat ?long
     ?currency ?gdpPppPerCapita ?giniCoefficient WHERE {

  SERVICE &lt;https://dbpedia.org/sparql&gt; {

    ?db_country_uri dbo:capital ?db_capital.

    BIND(?db_country_uri AS ?country_uri).
    BIND(str(?db_country_uri) AS ?country_uri_str).
    BIND(?db_capital AS ?capital).

#    OPTIONAL {
#      ?country_uri dbp:areaKm          ?area.          
#      ?country_uri dbo:populationTotal ?populationTotal.
#      ?country_uri geo:lat             ?lat.            
#      ?country_uri geo:long            ?long.
#    }

    OPTIONAL { ?country_uri dbp:areaKm          ?area.            }
    OPTIONAL { ?country_uri dbo:populationTotal ?populationTotal. }
    OPTIONAL { ?capital     geo:lat             ?lat.             }
    OPTIONAL { ?capital     geo:long            ?long.            }
    
    OPTIONAL { ?country_uri dbp:currencyCode    ?currency.        }
    OPTIONAL { ?country_uri dbp:gdpPppPerCapita ?gdpPppPerCapita.  }

    OPTIONAL { ?country_uri dbo:giniCoefficient ?giniCoefficient. }
  }

  ?esc_entrant_uri gsdlextracted:CountryDBURI ?country_uri_str.

}
ORDER BY ASC(?country_uri)  
</xsl:text>
	    </div>

	  
	    
	    <ul class="ssq">
	      <li>
		<b>Year Restricted (raw data rows):</b><br />
		<button type="button" class="load-ssq" id="load-ssq-restrict-year" onclick="ssq_load('ssq-restrict-year')">Load query above</button>
		<button type="button" class="exec-ssq" id="exec-ssq-restrict-year" onclick="ssq_execute()">Get Results</button><br />
		<p>Similar to the default query (all results returned) but restricted to entries associated with the year 2000.</p>
	      </li>

	      <div id="raw-sparql-show-more" class="showmore">
		<style>
		  #showmore-button-raw-sparql-show-more { width: 260px; }
		</style>
		
		<li>
		  <b>Year Restricted (raw data rows) then filter out <i>gsdlextracted:JSON</i> entries:</b><br/>
		  <button type="button" class="load-ssq" id="load-ssq-restrict-year-then-filter" onclick="ssq_load('ssq-restrict-year-then-filter')">Load query above</button>
		  <button type="button" class="exec-ssq" id="exec-ssq-restrict-year-then-filter" onclick="ssq_execute()">Get Results</button><br/>
		  <p>
		    Similar to the previous query (restricted to results from 2000) but filtering out the voluminous, but not very interesting to look at,
		    <i>gsdlextracted:JSON</i>.
		  </p>
		  <p>
		    This field is formed by Greenstone at ingest-time: it represents all the metadata fields for a particular country
		    stored in the JSON format.  It is a handy item for the Greenstone3 runtime-system to access, but in the context of viewing the output
		    of SPARQL queries at HTML only serves to pollute the generated result set with large text items.
		  </p>
		</li>
		
		<li>
		  <b>Country and Year Restricted (raw data rows):</b><br/>
		  <button type="button" class="load-ssq" id="load-ssq-restrict-country-and-year" onclick="ssq_load('ssq-restrict-country-and-year')">Load query above</button>
		  <button type="button" class="exec-ssq" id="exec-ssq-restrict-country-and-year" onclick="ssq_execute()">Get Results</button><br/>
		  
		  <p>Similar to the previous query, but restricted to entries about the country France, in the years 1996-2000.</p>
		</li>
	      </div>
	      <gsf:script>
		$('#raw-sparql-show-more').showMore({
	          minheight: 0,
	  	  buttontxtmore:"show further raw-data queries ...",
		  buttontxtless:"... show less"
	        });
	      </gsf:script>	    

	      <li>
		<b>Eurovision Song Contest (ESC) entrants:</b><br/>
		<button type="button" class="load-ssq" id="load-ssq-esc-entrants" onclick="ssq_load('ssq-esc-entrants')">Load query above</button>
		<button type="button" class="exec-ssq" id="exec-ssq-esc-entrants" onclick="ssq_execute()">Get Results</button><br/>
		
		<p>List all the entrants (country and year) sorted by year, then by country.</p>
	      </li>

	      <li>
		<b>Voting Totals in 1975:</b><br/>
		<button type="button" class="load-ssq" id="load-ssq-jury-votes-totals-1975" onclick="ssq_load('ssq-jury-votes-totals-1975')">Load query above</button>
		<button type="button" class="exec-ssq" id="exec-ssq-jury-votes-totals-1975" onclick="ssq_execute()">Get Results</button><br/>
		
		<p>In 1975 all voting was done exclusively by jury.  This query retrieves the
		totals of this Single voting format, and orders them by that score.</p>
	      </li>
	      
	      <li>
		<b>Jury Votes Cast by Country in 1975:</b><br/>
		<button type="button" class="load-ssq" id="load-ssq-jury-votes-1975" onclick="ssq_load('ssq-jury-votes-1975')">Load query above</button>
		<button type="button" class="exec-ssq" id="exec-ssq-jury-votes-1975" onclick="ssq_execute()">Get Results</button><br/>
		
		<p>List all votes case in the 1975 competition.</p>
	      </li>


	      <li>
		<b>Most points won:</b><br/>
		<button type="button" class="load-ssq" id="load-ssq-winning-totals" onclick="ssq_load('ssq-winning-totals')">Load query above</button>
		<button type="button" class="exec-ssq" id="exec-ssq-winning-totals" onclick="ssq_execute()">Get Results</button><br/>
		
		<p>List the countries which won and their voting totals, sorted by total score.</p>
	      </li>

	      <li>
		<b>Number of Times Countries Have Won:</b><br/>
		<button type="button" class="load-ssq" id="load-ssq-winning-freq-count" onclick="ssq_load('ssq-winning-freq-count')">Load query above</button>
		<button type="button" class="exec-ssq" id="exec-ssq-winning-freq-count" onclick="ssq_execute()">Get Results</button><br/>

		<p>
		  The number of times a country has won Eurovision across the years.  The includes
		  the years when only Juries voted (1956-2000) through to the introduction
		  of Televotes, where a variety of forms have been used such as only
		  Televotes, a pre-combined score based on Televotes and Jury votes, to
		  (from 2016 onwards) where the Jury and Tele votes are explicitly
		  given individually per country, then combined.
		</p>
	      </li>
	      

	      <li>
		<b>Facts and Figures About Competing Countries:</b><br/>
		<button type="button" class="load-ssq" id="load-ssq-country-info" onclick="ssq_load('ssq-country-info')">Load query above</button>
		<button type="button" class="exec-ssq" id="exec-ssq-country-info" onclick="ssq_execute()">Get Results</button><br/>
		
		<p>
		  Demographics about other related information about the countries the song contest entries are from.
		  Query takes a bit longer to run than the previous ones.
		</p>
	      </li>
	      
	    </ul>

	    <p>
	      Looking for something a bit more visual?  Why not head over to our 
	      <a href="{$library_name}/collection/{$collName}/page/sgvizler"><i>SGVizler page</i></a> and try the SPARQL-based visualization samples there.
	    </p>
	    
	  </div>
	</xsl:template>
	

</xsl:stylesheet>  

